set terminal postscript eps enhanced color solid size 7.7,9 'Times-Roman' 30 linewidth 2.5
set output 'lightcurve_optObsNuSTAR.eps'
#
#
#
# Set top and bottom margins to 0 so that there is no space between plots.
# Fix left and right margins to make sure that the alignment is perfect.
# Turn off xtics for all plots except the bottom one.
# In order to leave room for axis and tic labels underneath, we ask for
# a 4-plot layout but only use the top 3 slots.
#
set tmargin 0
set bmargin 0
set lmargin 8
set rmargin 1
unset xtics

set multiplot layout 3,1 title ''

unset title
#
#
t0=2459377.6903
set key at graph 0.25, 0.2
# same x axis range for all graphs - should go before the first plot
set xrange [11.2:12.3]
Theads = 'heads size 0.01,90 front linewidth 2'
set arrow from 0.3+2459388.975000-t0,12.0 to 0.3+2459388.975000-t0+0.153,12.0 @Theads
set label 'P_{orb}' at 0.3+2459388.975000-t0+0.153/2,12.0-0.025 center font 'Times-Roman,30'
#
# Color scheme
load 'my.pal'
#
# NuSTAR
set format y '%1g'
set ylabel 'Count rate (cts/s)'
plot \
'nu90701321002A01_srMbg.txt' u ( ($1)/86400+2459388.975000-t0):2:3 w e pt 7 ps 1.5 lc 'black' title 'FPMA', \
'nu90701321002B01_srMbg.txt' u ( ($1)/86400+2459388.975000-t0):2:3 w e pt 7 ps 1.5 lc 'red' title 'FPMB'
#
# same x axis labeling for all graphs - should go after the last plot
set xtics nomirror
set xlabel 'Days since t_0'
#set key top left
set key at graph 0.25, 0.627
# Optical
# y axis
set ytics mirror
set ylabel 'V (mag.)'
set ytics 0.1
set format y '%4.1f'
set yrange [12.05:11.55]
plot \
'aavso_V_BVS.txt'  using ($1-t0):2:3 w e ls 6 pt 7 ps 1.0 title 'BVS', \
'aavso_V_DEY.txt'  using ($1-t0):2:3 w e ls 7 pt 7 ps 1.0 title 'DEY', \
'aavso_V_MZK.txt'  using ($1-t0):2:3 w e ls 8 pt 7 ps 1.0 title 'MZK', \
'aavso_V_RFDA.txt' using ($1-t0):2:3 w e ls 4 pt 7 ps 1.0 title 'RFDA', \
'aavso_V_RHM.txt'  using ($1-t0):2:3 w e ls 5 pt 7 ps 1.0 title 'RHM', \
'aavso_V_UJHA.txt' using ($1-t0):2:3 w e ls 1 pt 7 ps 1.0 title 'UJHA', \
'aavso_V_VMT.txt'  using ($1-t0):2:3 w e ls 2 pt 7 ps 1.0 title 'VMT', \
'aavso_V_WGUA.txt' using ($1-t0):2:3 w e ls 3 pt 7 ps 1.0 title 'WGUA'
unset label
#
unset multiplot
! convert -density 300 lightcurve_optObsNuSTAR.eps  -background white -alpha remove lightcurve_optObsNuSTAR.png
#
