# 7,4
set terminal postscript eps enhanced color solid size 7.7,9 'Times-Roman' 30 linewidth 2.5
set output 'lightcurve_opticalVLA.eps'
#
#
#
# Set top and bottom margins to 0 so that there is no space between plots.
# Fix left and right margins to make sure that the alignment is perfect.
# Turn off xtics for all plots except the bottom one.
# In order to leave room for axis and tic labels underneath, we ask for
# a 4-plot layout but only use the top 3 slots.
#
set tmargin 0
set bmargin 0
set lmargin 7
set rmargin 1
unset xtics
#unset ytics

set multiplot layout 4,1 title ''

#set key autotitle column nobox samplen 1 noenhanced
unset title
#
#
t0=2459377.6903
set key top right
# same x axis range for all graphs - should go before the first plot
set xrange [0.005:2000]
set logscale x
#y axis
set ytics mirror
set ylabel 'Optical brightness (mag.)'
#set yrange [] reverse
set yrange [18.5:5.5]
set format y '%3.0f'
#set format y '%6.0f'
#
#set tics out
#
Theads = 'heads size 10,90 front linewidth 2'
set arrow from 2459377.94036-t0,17 to 2459378.69036-t0,17 @Theads
set label 'Fermi' at (2459377.94036+2459378.69036-2*t0)/2-0.1,17+0.75 center font 'Times-Roman,30'
#
set arrow from 2459388.975000-t0,17 to 2459389.933333-t0,17 @Theads
set label 'NuSTAR' at (2459388.975000+2459389.933333-2*t0)/2,17-1.0 center font 'Times-Roman,30'
#
set arrow from 18.9,17 to 67,17 @Theads
set label 'SSS' at (18.9+67)/2-8,17+0.75 center font 'Times-Roman,30'
#
# Color scheme
load 'dark2.pal'
#
# Optical
plot \
'aavso_Vis_everyone_edit.txt' \
using ($1-t0):2 pt 7 ps 1.0 lc 'dark-grey' title 'AAVSO visual', \
'aavso_V_everyone_edit.txt' \
using ($1-t0):2 ls 1 pt 7 ps 1.0 title 'AAVSO V', \
'aavso_CV_everyone_edit.txt' \
using ($1-t0):2 ls 5 pt 7 ps 1.0 title 'AAVSO CV', \
'asassn_g_edit.dat' \
using ($1-t0):2 ls 2 pt 7 ps 1.0 title 'ASAS-SN g', \
'evryscope_lc.txt' \
using ($1-t0+2400000.5):2 ls 4 pt 7 ps 1.0 title 'Evryscope g', \
'asc_stacked_lc.txt' \
using ($1-t0+2400000.5):2 ls 3 pt 7 ps 1.0 title 'MLO-ASC'
#
#unset arrow
unset label
# Radio
set key bottom left
set yrange [0.02:20]
set logscale y
set format y '%1g'
set ylabel 'Radio F_{/Symbol n} (mJy)'
#load 'moreland.pal'
load 'paired.pal'
plot \
0.003*x**2 lc 'black' lw 2 dashtype 2 title 'F_{/Symbol n} \~ t^2', \
 'vlalightcurve_2.6GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 1 pt  7 ps 2 title '2.6 GHz', \
 'vlalightcurve_2.6GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 1 pt  7 ps 2 title '', \
 'vlalightcurve_UL_2.6GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 1 pt 11 ps 2.5 title '', \
 'vlalightcurve_3.4GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 2 pt  7 ps 2 title '3.4 GHz', \
 'vlalightcurve_3.4GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 2 pt  7 ps 2 title '', \
 'vlalightcurve_UL_3.4GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 2 pt 11 ps 2.5 title '', \
 'vlalightcurve_5.1GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 3 pt  7 ps 2 title '5.1 GHz', \
 'vlalightcurve_5.1GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 3 pt  7 ps 2 title '', \
 'vlalightcurve_UL_5.1GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 3 pt 11 ps 2.5 title '', \
 'vlalightcurve_7.0GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 4 pt  7 ps 2 title '7.0 GHz', \
 'vlalightcurve_7.0GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 4 pt  7 ps 2 title '', \
 'vlalightcurve_UL_7.0GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 4 pt 11 ps 2.5 title '', \
'vlalightcurve_13.7GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 5 pt  7 ps 2 title '13.7 GHz', \
'vlalightcurve_13.7GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 5 pt  7 ps 2 title '', \
'vlalightcurve_UL_13.7GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 5 pt 11 ps 2.5 title '', \
'vlalightcurve_16.5GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 6 pt  7 ps 2 title '16.5 GHz', \
'vlalightcurve_16.5GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 6 pt  7 ps 2 title '', \
'vlalightcurve_UL_16.5GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 6 pt 11 ps 2.5 title '', \
'vlalightcurve_31.1GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 7 pt  7 ps 2 title '31.1 GHz', \
'vlalightcurve_31.1GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 7 pt  7 ps 2 title '', \
'vlalightcurve_UL_31.1GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 7 pt 11 ps 2.5 title '', \
'vlalightcurve_34.9GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 8 pt  7 ps 2 title '34.9 GHz', \
'vlalightcurve_34.9GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 8 pt  7 ps 2 title '', \
'vlalightcurve_UL_34.9GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 8 pt 11 ps 2.5 title ''
#
# same x axis labeling for all graphs - should go after the last plot
set xtics nomirror
set xlabel 'Days since t_0'
# Override automatic calculation of lower plot boundary
#set bmargin at screen 0.07
#
# Radio Tb
#set format y '%2h'
set format y '10^{%L}'
set ylabel 'T_b (K)'
#unset yrange
set yrange [10:9e6]
plot \
 'vlalightcurve_2.6GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 1 pt 7 ps 2 title '2.6 GHz', \
 'vlalightcurve_3.4GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 2 pt 7 ps 2 title '3.4 GHz', \
 'vlalightcurve_5.1GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 3 pt 7 ps 2 title '5.1 GHz', \
 'vlalightcurve_7.0GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 4 pt 7 ps 2 title '7.0 GHz', \
'vlalightcurve_13.7GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 5 pt 7 ps 2 title '13.7 GHz', \
'vlalightcurve_16.5GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 6 pt 7 ps 2 title '16.5 GHz', \
'vlalightcurve_31.1GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 7 pt 7 ps 2 title '31.1 GHz', \
'vlalightcurve_34.9GHz.txt' u ($1+2400000.5-2459377.6903):4 ls 8 pt 7 ps 2 title '34.9 GHz'
unset multiplot
! convert -density 300 lightcurve_opticalVLA.eps  -background white -alpha remove lightcurve_opticalVLA.png
#
# Make a linear-scale lightcurveplot
# 7,4
set terminal postscript eps enhanced color solid size 7,4 'Times-Roman' 24 linewidth 2
set output 'lightcurve_VLAlinear.eps'
unset tmargin 
unset bmargin 
unset lmargin 
unset rmargin 
unset logscale
unset yrange
set xrange [0:150]
set format y '%1g'
set ylabel 'F_{/Symbol n} (mJy)'
set key top right
plot \
 'vlalightcurve_2.6GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 1 pt  7 ps 2 title '2.6 GHz', \
 'vlalightcurve_2.6GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 1 pt  7 ps 2 title '', \
 'vlalightcurve_UL_2.6GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 1 pt 11 ps 2.5 title '', \
 'vlalightcurve_3.4GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 2 pt  7 ps 2 title '3.4 GHz', \
 'vlalightcurve_3.4GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 2 pt  7 ps 2 title '', \
 'vlalightcurve_UL_3.4GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 2 pt 11 ps 2.5 title '', \
 'vlalightcurve_5.1GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 3 pt  7 ps 2 title '5.1 GHz', \
 'vlalightcurve_5.1GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 3 pt  7 ps 2 title '', \
 'vlalightcurve_UL_5.1GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 3 pt 11 ps 2.5 title '', \
 'vlalightcurve_7.0GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 4 pt  7 ps 2 title '7.0 GHz', \
 'vlalightcurve_7.0GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 4 pt  7 ps 2 title '', \
 'vlalightcurve_UL_7.0GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 4 pt 11 ps 2.5 title '', \
'vlalightcurve_13.7GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 5 pt  7 ps 2 title '13.7 GHz', \
'vlalightcurve_13.7GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 5 pt  7 ps 2 title '', \
'vlalightcurve_UL_13.7GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 5 pt 11 ps 2.5 title '', \
'vlalightcurve_16.5GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 6 pt  7 ps 2 title '16.5 GHz', \
'vlalightcurve_16.5GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 6 pt  7 ps 2 title '', \
'vlalightcurve_UL_16.5GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 6 pt 11 ps 2.5 title '', \
'vlalightcurve_31.1GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 7 pt  7 ps 2 title '31.1 GHz', \
'vlalightcurve_31.1GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 7 pt  7 ps 2 title '', \
'vlalightcurve_UL_31.1GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 7 pt 11 ps 2.5 title '', \
'vlalightcurve_34.9GHz.txt' u ($1+2400000.5-2459377.6903):2                     ls 8 pt  7 ps 2 title '34.9 GHz', \
'vlalightcurve_34.9GHz.txt' u ($1+2400000.5-2459377.6903):2:3 with errorbars    ls 8 pt  7 ps 2 title '', \
'vlalightcurve_UL_34.9GHz.txt' u ($1+2400000.5-2459377.6903):2                  ls 8 pt 11 ps 2.5 title ''
! convert -density 300 lightcurve_VLAlinear.eps  -background white -alpha remove lightcurve_VLAlinear.png
