
#include "pfi.h"

#include "synchrotron_cloud_fit.h"

#include <math.h>

int main(int argc, char **argv){
 FILE *inputfile;
 double freq[MAX_NUMBER_OF_FREQUENCIES];
 float flogfreq[MAX_NUMBER_OF_FREQUENCIES];
 double logfreq[MAX_NUMBER_OF_FREQUENCIES];
 double flux[MAX_NUMBER_OF_FREQUENCIES];
 float flogflux[MAX_NUMBER_OF_FREQUENCIES];
 double logflux[MAX_NUMBER_OF_FREQUENCIES];
 double flux_sigma[MAX_NUMBER_OF_FREQUENCIES];
 float fflux_sigma[MAX_NUMBER_OF_FREQUENCIES];
 double w[MAX_NUMBER_OF_FREQUENCIES];
 int i=0;
 int j=0;
 
 double A,B,sumsqres,TMP,cov00,cov01,cov11;
 
 float curX,curY;
 char curC;
 float curveX[1000];
 float curveY[1000];
 
 double best_gamma,best_numax,best_imax;
 double sync_freq[1000],sync_flux[1000];
 
 float win_x_min,win_x_max,win_y_min,win_y_max,win_sum;
 
 double max_chi_square;
 double chi_square;

 /* For legend */
 static const float echsize=0.8f; /* Char height of extra anotation text */
 //static const float sepinc=0.3f;  /* Separation between extra anotation lines */
 static const float sepinc=0.7f;  /* Separation between extra anotation lines */
 char str1[1024];

 char input_file_name[1024];
 char date_string[1024];


 int synchrotron_bad_fit=0;


 if( argc>1 ) {
  strncpy(input_file_name,argv[1],1024);
  input_file_name[1024-1]='\0'; // just in case
  sprintf(str1,"cp -v %s test.dat",input_file_name);
  system(str1);
  strncpy(date_string,input_file_name,1024);
  for(i=0;i<strlen(date_string);i++) {
   if( date_string[i] == '.' ) {
    date_string[i]='\0';
    break;
   }
  }
 } else {
  date_string[0]='\0';
 }
 
 /* Read input data */
 system("sort -n test.dat > test.tmp; mv -f test.tmp test.dat");
 inputfile=fopen("test.dat","r");
 i=0;
 while(-1<fscanf(inputfile,"%lf %lf %lf", &freq[i], &flux[i], &flux_sigma[i])){
 //while(3<fscanf(inputfile,"%lf %lf %lf", &freq[i], &flux[i], &flux_sigma[i])){
  if( 0!=isnan(freq[i]) ){
   continue;
  }
  logfreq[i]=log10(freq[i]);
  flogfreq[i]=(float)logfreq[i];
  logflux[i]=log10(flux[i]);
  flogflux[i]=(float)logflux[i];
  fflux_sigma[i]=(float)(log10(flux[i]+flux_sigma[i])-log10(flux[i]-flux_sigma[i]))/2.0;
//  fprintf(stderr,"%lf %lf %lf\n",flogfreq[i],flogflux[i],fflux_sigma[i]);
//  w[i]=1.0;// /(flux_sigma[i]*flux_sigma[i]);
//  w[i]=2.0/(flux_sigma[i]*flux_sigma[i]+flux_sigma[i]*flux[i]*freq[0]/freq[i]);
//  w[i]=2.0/(flux_sigma[i]*flux_sigma[i]+flux_sigma[i]*flux[i]);
  w[i]=1.0/(flux_sigma[i]*flux_sigma[i]+(0.05*flux[i])*(0.05*flux[i]));
  fprintf(stderr,"freq = %8.4lf  flux = %8.4lf  old sigma =%8.4lf  ", freq[i], flux[i], flux_sigma[i]);
  flux_sigma[i]=1.0/sqrt(w[i]);
  fprintf(stderr,"new sigma = %8.4lf\n",flux_sigma[i]);
  
  // positive test
  //if( flux[i]<0.0 )i--;
  
  i++;
  
 }
 fclose(inputfile);

 /* Shamanism */
 for(j=0;j<i;j++){
  fprintf(stderr,"considering %lf w_old=%lf ",freq[j],w[j]);
  if( j!=0 && j!=i-1 )
   if ( fabs(logfreq[j]-logfreq[j-1])<LOG_FREQ_BIN ||  fabs(logfreq[j]-logfreq[j+1])<LOG_FREQ_BIN )w[j]=w[j]/2;
  if( j==0 )
   if ( fabs(logfreq[j]-logfreq[j+1])<LOG_FREQ_BIN )w[j]=w[j]/2;
  if( j==i-1 )
   if ( fabs(logfreq[j]-logfreq[j-1])<LOG_FREQ_BIN )w[j]=w[j]/2;
  fprintf(stderr,"w_new=%lf\n",w[j]);
 }
 for(j=0;j<i;j++)w[j]=w[j]*2; //!!!

 
 /* Warinig! pfi.c uses a separate liner fitting function */
 gsl_fit_wlinear(logfreq,1,w,1,logflux,1,i,&B,&A,&cov00,&cov01,&cov11,&sumsqres);
 fprintf(stderr,"A=%lf +/-%lf B=%lf +/-%lf\n",A,sqrt(cov11),B,sqrt(cov00));
 gsl_fit_linear(logfreq,1,logflux,1,i,&B,&A,&cov00,&cov01,&cov11,&sumsqres);
 fprintf(stderr,"A=%lf +/-%lf B=%lf +/-%lf (no weights)\n",A,sqrt(cov11),B,sqrt(cov00));
 powerfit( &B, &A, freq, flux, w, i, &max_chi_square, 1);//B=log10(B);
 fprintf(stderr, "\nA= %lf (power law fit)\n\n",A);
// A=-0.550000;B=-0.350000;
// fprintf(stderr,"A=%lf  B=%lf\n",A,B);

 fprintf(stderr,"pgsimplefit main: max_chi_square=%lf\n",max_chi_square);
 
 /* Fit synchrotron spectrum */
 synchrotron_bad_fit=fit_synchrotron( &best_gamma, &best_numax, &best_imax, freq, flux, w, i, max_chi_square, 1, &chi_square);
 if( 1==test_drop_highest_freqency( &best_numax, freq, flux, w, &i ) ){
  powerfit( &B, &A, freq, flux, w, i, &max_chi_square, 1);//recompute liner fit too!!!
  fprintf(stderr, "\nA= %lf (power law fit)\n\n",A);
  synchrotron_bad_fit=fit_synchrotron( &best_gamma, &best_numax, &best_imax, freq, flux, w, i, max_chi_square, 1, &chi_square);
 }

 fprintf(stderr,"\nFinal fit_synchrotron() return status: synchrotron_bad_fit=%d\n",synchrotron_bad_fit);
 
 fprintf(stderr,"pgsimplefit main: chi_square=%lf\n",chi_square);
 
 /* Compute window */
 win_x_min=-1.0;win_x_max=2.0;win_y_min=-2.5;win_y_max=0.5;
 win_sum=0;
 for(j=0;j<i;j++){
  // positive test
  if( flux[j]>0.0 )win_sum+=logflux[j];
 }
 win_sum=win_sum/i;
 win_y_min=win_sum-1.5;
 win_y_max=win_sum+1.5;

 win_sum=0;
 for(j=0;j<i;j++){
  win_sum+=logfreq[j];
 }
 win_sum=win_sum/i;
 win_x_min=win_sum-1.5;
 win_x_max=win_sum+1.5;


 //curC='A';
 curC='S';

 while( curC!='X' && curC!='x' ){
  /* Start GUI */
  if( curC=='S' || curC=='s' )
   cpgbeg(0, "/VCPS", 1, 1);
  else{
   cpgbeg(0, "/XW", 1, 1);
   cpgscr( 0, 0.10, 0.31, 0.32);  /* set default vast window background */
  }
 
  cpgpap( 0.0, 1.0); /* Make square plot */ 
//  cpgsvp(0.2375, 0.7625, 0.15, 0.85);

  cpgpage();
  if( curC!='S' && curC!='s' )cpgscr( 0, 0.0, 0.0, 0.0);  /* set black background */

  cpgsvp(0.15, 0.95, 0.1, 0.9);
  //cpgswin(win_x_min,win_x_max,win_y_min,win_y_max); /* Set window */
  cpgswin( log10(0.3), log10(300), log10(0.009), log10(9));

  /* Paint (your wagon) the background */
  if( curC!='S' && curC!='s' ){
   cpgscr( 0, 0.08, 0.08, 0.09);  /* set background */
   cpgsci(0);
   cpgrect(win_x_min,win_x_max,win_y_min,win_y_max);
   cpgsci(1);
  }
  
  cpgscf(2); /* Set roman font */
  //cpgsch(1.3);
  double axisoffset=0.07;
  cpgsch(1.5);
  cpgslw(3);
  cpgbox("BCLNST1", 0.0 , 10, "BCLNSTV1", 0.0, 10);
  //cpgmtxt("B",2.5,0.5,0.5,"Frequency (GHz)");
  cpgmtxt("B",2.2,0.5,0.5,"Frequency (GHz)");
  cpgmtxt("L",3.2,0.5,0.5,"Flux Density (mJy)");
  //cpgmtxt("L",3.2,0.5,0.5,"Flux Density (\gmJy)");

  /* Draw best fit synchrotron spectrum */
  if( synchrotron_bad_fit==0 ){
   sync_freq[0]=0.2;
   sync_freq[999]=40;
   for(j=1;j<999;j++){
    sync_freq[j]=pow(10,log10(sync_freq[j-1])+(log10(sync_freq[999])-log10(sync_freq[0]))/1000); 
   }
   synchrotron( best_gamma, best_numax, best_imax, sync_freq, sync_flux, 1000);
   for(j=0;j<1000;j++){
    curveX[j]=(float)log10(sync_freq[j]);
    curveY[j]=(float)log10(sync_flux[j]);
   }
   cpgsci(4); // 
   cpgline(1000, curveX, curveY);
  }
 
  /* Draw best fit line */ 
  curveX[0]=flogfreq[0];
  curveY[0]=A*curveX[0]+B;
  curveX[1]=flogfreq[i-1];
  curveY[1]=A*curveX[1]+B;
  cpgsci(3); // ̣
  cpgline(2, curveX, curveY);
  
  /* Draw legend */
  cpgsci(1); // while
  sprintf(str1,"Power-law");
  //cpgmtxt("T",-2.4*echsize,0.07,0.0,str1);
  cpgmtxt("T",-2.4*echsize,axisoffset,0.0,str1);
  sprintf(str1,"\\ga = %.1lf",A);
  //cpgmtxt("T",-2.4*echsize-echsize-sepinc,0.07,0.0,str1);
  cpgmtxt("T",-2.4*echsize-echsize-sepinc,axisoffset,0.0,str1);
  sprintf(str1,"\\gx\\u2\\b\\d\\dred\\u = %.1lf",max_chi_square/(i-2));
  //cpgmtxt("T",-2.4*echsize-2*(echsize+sepinc),0.07,0.0,str1);
  
  // Observing date from file name
  sprintf(str1,date_string);
  cpgmtxt("B",-2.4*echsize,0.93,1.0,str1);
  
  // Manually put t_0 + Xd labels
  if ( 0 == strncmp("2021-06-16",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 4.2\\ud\\d");
  }
  if ( 0 == strncmp("2021-06-17",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 5.2\\ud\\d");
  }
  if ( 0 == strncmp("2021-06-21",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 9.2\\ud\\d");
  }
  if ( 0 == strncmp("2021-06-22",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 10.1\\ud\\d");
  }
  if ( 0 == strncmp("2021-06-25",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 13.1\\ud\\d");
  }
  if ( 0 == strncmp("2021-06-27",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 15.2\\ud\\d");
  }
  if ( 0 == strncmp("2021-07-13",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 31.0\\ud\\d");
  }
  if ( 0 == strncmp("2021-07-26",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 44.1\\ud\\d");
  }
  if ( 0 == strncmp("2021-08-25",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 74.0\\ud\\d");
  }
  if ( 0 == strncmp("2021-09-03",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 83.0\\ud\\d");
  }
  if ( 0 == strncmp("2021-09-11",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 91.0\\ud\\d");
  }
  if ( 0 == strncmp("2021-11-01",date_string,10) ) {
   cpgmtxt("B",-2.4*echsize-echsize-sepinc,0.93,1.0,"t\\d0\\u + 142.8\\ud\\d");
  }
  

  if( synchrotron_bad_fit==0 ){
   sprintf(str1,"Synchrotron slab");
   //cpgmtxt("T",-2.4*echsize,0.93,1.0,str1);
   cpgmtxt("T",-2.4*echsize,1.0-axisoffset,1.0,str1);
   sprintf(str1,"\\gn\\dpeak\\u = %.1lf GHz",best_numax);
 //  sprintf(str1,"\\ga\\dsync\\u = %.1lf",(1-best_gamma)/2);
 //  sprintf(str1,"S\\d\\gn\\d0\\u\\u = %.3lf",best_imax);
   //cpgmtxt("T",-2.4*echsize-echsize-sepinc,0.93,1.0,str1);
   cpgmtxt("T",-2.4*echsize-echsize-sepinc,1.0-axisoffset,1.0,str1);
  // sprintf(str1,"\\gn\\d0\\u = %.1lf",best_numax);
  // sprintf(str1,"\\ga = %.1lf",(1-best_gamma)/2);
   ////sprintf(str1,"S\\dpeak\\u = %.2lf Jy",best_imax);
   //sprintf(str1,"S\\dpeak\\u = %.2lf mJy",best_imax);
   sprintf(str1,"F\\d\\gn peak\\u = %.2lf mJy",best_imax);
   //cpgmtxt("T",-2.4*echsize-2*(echsize+sepinc),0.93,1.0,str1);
   cpgmtxt("T",-2.4*echsize-2*(echsize+sepinc),1.0-axisoffset,1.0,str1);
  // sprintf(str1,"\\ga = %.1lf",(1-best_gamma)/2);
 //  sprintf(str1,"\\gn\\dpeak\\u = %.2lf GHz",best_numax);
   sprintf(str1,"\\ga\\dsync\\u = %.1lf",(1-best_gamma)/2);
   //cpgmtxt("T",-2.4*echsize-3*(echsize+sepinc),0.93,1.0,str1);
   cpgmtxt("T",-2.4*echsize-3*(echsize+sepinc),1.0-axisoffset,1.0,str1);
   sprintf(str1,"\\gx\\u2\\b\\d\\dred\\u = %.1lf",chi_square/(i-3));
   //cpgmtxt("T",-2.4*echsize-4*(echsize+sepinc),0.95,1.0,str1);
  }
//  fprintf(stderr,"X^2 %lf  %lf",max_chi_square, chi_square);
  
  /* Draw points */
  cpgsci(2); // 
  for(j=0;j<i;j++){
   // positive test
   if( flux[j]>0.0 )
    cpgerr1(6, flogfreq[j], (float)(log10(flux[j]+flux_sigma[j])+log10(flux[j]-flux_sigma[j]))/2.0, fflux_sigma[j], 1.0);
 //  fprintf(stderr,"%lf %lf %lf\n",flogfreq[j],(float)(log10(flux[j]+flux_sigma[j])+log10(flux[j]-flux_sigma[j]))/2.0,fflux_sigma[j]);
  }
  cpgpt(i, flogfreq, flogflux, 17);
   
  if( curC!='S' && curC!='s' )
   cpgcurs( &curX, &curY, &curC);
  else{
   cpgend();
   if( strlen(date_string)>1 ) {
    sprintf(str1,"mv -v pgplot.ps %s.ps && ps2eps --ignoreBB --force %s.ps",date_string,date_string);
    system(str1);
   }
   return 0;
  }
 }
 return 0;
}
