#include <stdio.h>
#include <math.h>
#include <stdlib.h>
//#include <gsl/gsl_statistics.h>
#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_sf_gamma.h>

#include <time.h> /* to measure execution time */

#define MAX_NUMBER_OF_FREQUENCIES 100

#define BIG_STEP_TO_SMALL_STEP 5
//#define BIG_STEP_TO_SMALL_STEP 10


//#define MIN_PEAK_FREQ 0.3
//#define MAX_PEAK_FREQ 5
//#define PEAK_FREQ_STEP 0.01
#define PEAK_FREQ_STEP 0.1

//#define MAX_OBS_FLUX_TO_MAX_PEAK_FLUX 2.0
//#define MIN_OBS_FLUX_TO_MIN_PEAK_FLUX 3.0

#define MAX_OBS_FLUX_TO_MAX_PEAK_FLUX 1.0
#define MIN_OBS_FLUX_TO_MIN_PEAK_FLUX 1.0


#define MAX_PEAK_FLUX 5.0
//#define PEAK_FLUX_STEP 0.004
#define PEAK_FLUX_STEP 0.002
//#define PEAK_FLUX_STEP 0.02

/* GAMMA=2*ALPHA+1 */
//#define MIN_GAMMA 0.0
#define MIN_GAMMA 0.34
#define MAX_GAMMA 5.0
//#define GAMMA_STEP 0.01
#define GAMMA_STEP 0.01

/* powerfit() */
//#define SP_INDEX_MAX 1.5
#define SP_INDEX_MAX 3.0
#define SP_INDEX_MIN -3.0
//#define SP_INDEX_STEP 0.01
#define SP_INDEX_STEP 2*GAMMA_STEP

#define C_MAX  20.0
#define C_MIN -20.0
//#define C_MAX  2.0
//#define C_MIN -2.0
#define C_STEP 0.01


int powerfit( double *best_C, double *best_A, double *freq, double *observed_flux, double *w, int N_freq, double *max_chi_square, int verbose);

void init_random_generator(){
 FILE *randomfile;
 int c;
 randomfile=fopen("/dev/urandom","r");
 c=fgetc(randomfile);
 fclose(randomfile);
 srandom((unsigned int)c);
 return;
}

/*
double isolve(double x){
 float tau;  
 float gamma;
 float gamma_p4_d5;
 int i; 
 tau=(float)x;  
 gamma=(float)x;
 gamma_p4_d5=(gamma+4)/5;
 for(i=0;i<100;i++){
  tau=logf(1+gamma_p4_d5*tau);
 }
 return (double)tau;
}

double J(double z, double gamma){
 double res;
 res=pow(z,-1*(gamma+4)/2);
 res=pow(z,2.5)*(1-exp(-1*res));
 return res;
}

void synchrotron(double gamma, double numax, double Imax, double *freq, double *theor_flux, int N_freq){
 double taumax;
 double nu1;   
 double S_nu1;
 int i;
 taumax=isolve(gamma);
 nu1=pow(taumax,2/(gamma+4))*numax;
 S_nu1=Imax/J(numax/nu1,gamma);
 for(i=0;i<N_freq;i++){
  theor_flux[i]=S_nu1*J(freq[i]/nu1,gamma);
 }
 return;
}
*/

float isolve(float x){
 float tau;  
 float old_tau=0.0;
 float gamma;
 float gamma_p4_d5;
 int i; 
 tau=x;  
 gamma=x;
 gamma_p4_d5=(gamma+4)/5;
 for(i=0;i<50;i++){
  tau=logf(1+gamma_p4_d5*tau);
  if( fabsf(old_tau-tau)<0.0001 )
   break;
  else
   old_tau=tau;
 }
 return tau;
}

float J(float z, float gamma){
 float res;
 res=powf(z,-1*(gamma+4)/2);
 res=powf(z,2.5)*(1-exp(-1*res));
 return res;
}

void synchrotron(double gamma, double numax, double Imax, double *freq, double *theor_flux, int N_freq){
 float fnumax=(float)numax;
 float fgamma=(float)gamma;
 float taumax;
 float nu1;   
 float S_nu1;
 int i;
 taumax=isolve(fgamma);
 nu1=powf(taumax,2/(fgamma+4))*fnumax;
 S_nu1=(float)Imax/J(fnumax/nu1,fgamma);
 for(i=0;i<N_freq;i++){
  theor_flux[i]=(double)(S_nu1*J((float)freq[i]/nu1,fgamma));
 }
 return;
}


int set_synchrotron_limits(double *best_gamma, double *best_numax, double *best_imax, double *freq, double *observed_flux, double *w, int N_freq){
 double sum_sq_res,best_sum_sq_res;
 double gamma,numax,imax;
 double theor_flux[MAX_NUMBER_OF_FREQUENCIES];
 int i;
 
 best_sum_sq_res=9999999;
 
 double var_max_freq=0.0;
 double var_min_freq=9999999;
          
 double alpha_power_law;
 double C;
// powerfit( &C, &alpha_power_law, freq, observed_flux, w, N_freq, &best_sum_sq_res, 0);

// !!! if( alpha_power_law>0.0 )return
 
 best_sum_sq_res=9999999;
           
 /* Choose min peak flux */
 double var_min_peak_flux=9999999;
 double var_max_peak_flux=0.0;
 for(i=0;i<N_freq;i++){
  if( observed_flux[i]<var_min_peak_flux )var_min_peak_flux=observed_flux[i];
  if( observed_flux[i]>var_max_peak_flux )var_max_peak_flux=observed_flux[i];
  if( freq[i]>var_max_freq )var_max_freq=freq[i];                            
  if( freq[i]<var_min_freq )var_min_freq=freq[i];
  //  fprintf(stderr,"%lf %lf\n",freq[i],observed_flux[i]);
 }
 numax=var_min_freq=var_min_freq/MIN_OBS_FLUX_TO_MIN_PEAK_FLUX;
 gamma=(*best_gamma);
// gamma=1-2*alpha_power_law;
// fprintf(stderr,"alpha_power_law=%lf gamma=%lf\n",alpha_power_law,gamma);
/* double numax_min=var_max_freq;
 double numax_max=var_min_freq;
*/  
// double imax_min=MAX_PEAK_FLUX;
// double imax_max=var_min_peak_flux;
    
/* double gamma_min=MAX_GAMMA;
 double gamma_max=MIN_GAMMA;
*/
 
 /* Choose best imax */     
// for(imax=var_min_peak_flux;imax<MAX_PEAK_FLUX;imax+=BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP){
 for(imax=var_min_peak_flux;imax<10*var_max_peak_flux;imax+=BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP){
  sum_sq_res=0;
  synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
  for(i=0;i<N_freq;i++){
   if( observed_flux[i]!=0.0 ){
    sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
    if(sum_sq_res>best_sum_sq_res)break;
   }
  }
  if( sum_sq_res<best_sum_sq_res ){
   best_sum_sq_res=sum_sq_res;
   (*best_imax)=imax;
  }
 }                     
 
 /* Second pass - fine tuning */
 for(imax=(*best_imax)-1*BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP;imax<(*best_imax)+1*BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP;imax+=PEAK_FLUX_STEP){
  sum_sq_res=0;
  synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
  for(i=0;i<N_freq;i++){
   if( observed_flux[i]!=0.0 ){
    sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
    if(sum_sq_res>best_sum_sq_res)break;
   }
  }
  if( sum_sq_res<best_sum_sq_res ){
   best_sum_sq_res=sum_sq_res;
   (*best_imax)=imax;  
  }
 }
                     
 return 0;
}

int test_drop_highest_freqency( double *best_numax, double *freq, double *observed_flux, double *w, int *N_freq ){
 int i;
 int i_highest_freq;
 double highest_freq=0.0;
 double lowest_freq=9999999999999.99;
// fprintf(stderr,"TEST test_drop_highest_freqency() %lf %d\n",(*best_numax),(*N_freq));
 for(i=0;i<(*N_freq);i++){
//  fprintf(stderr,"%d %lf %lf %lf\n",i,freq[i],observed_flux[i],w[i]);
  if( freq[i]>highest_freq ){
   highest_freq=freq[i];
   i_highest_freq=i;
  }
  if( freq[i]<lowest_freq )lowest_freq=freq[i];
 }
 if( lowest_freq>(*best_numax) ){
//  fprintf(stderr,"Drop highest freqency!\n");
  observed_flux[i_highest_freq]=0.0;
/*  for(i=i_highest_freq;i<(*N_freq)-1;i++){
   freq[i]=freq[i+1];
   observed_flux[i]=observed_flux[i+1];
   w[i]=w[i+1];
  }
  //(*N_freq)=(*N_freq);*/
  return 1;
 }
 return 0;
}

int fit_synchrotron_my(double *best_gamma, double *best_numax, double *best_imax, double *freq, double *observed_flux, double *w, int N_freq, double max_chi_square, int verbose, double *chi_square){
 FILE *a;
 int i;
 double min_freq,max_freq;

 // tests
 for(i=0;i<N_freq;i++){
  if( observed_flux[i]<0.0 )return 5;
 }

 min_freq=max_freq=freq[0];
 a=fopen("test.dat","w");
 for(i=0;i<N_freq;i++){
  fprintf(a,"%.5lf %.5lf %.5lf\n",freq[i],observed_flux[i],sqrt(1.0/w[i]));
  if( freq[i]<min_freq )min_freq=freq[i];
  if( freq[i]>max_freq )max_freq=freq[i];
 }
 fclose(a);
 system("perl spec_fit.pl test.dat > test.out");
 a=fopen("test.out","r");
 fscanf(a,"%lf %lf %lf",best_imax,best_numax,best_gamma);
 fclose(a);
 (*best_gamma)=1-2*(*best_gamma);// gamma=1-2*alpha

 (*chi_square)=1.0;//hack

 if( (*best_numax)<min_freq )return 1;
 if( (*best_numax)>max_freq )return 1;
 
 return 0;
}

int fit_synchrotron(double *best_gamma, double *best_numax, double *best_imax, double *freq, double *observed_flux, double *w, int N_freq, double max_chi_square, int verbose, double *chi_square){
// fprintf(stderr,"MAX_PEAK_FLUX=%lf\n",MAX_PEAK_FLUX);
 double sum_sq_res,best_sum_sq_res;
 double gamma,numax,imax;
 double theor_flux[MAX_NUMBER_OF_FREQUENCIES];
 int i;

 /* Execution time measurements */
 time_t start_time;
 time_t end_time;
 double elapsed_time;

 /* Start timer */
 start_time=time(NULL);                                
     
 best_sum_sq_res=9999999;
/*
 fprintf(stderr,"fit_synchrotron(): recived max_chi_square=%lf\n",max_chi_square);
 if( max_chi_square<=0.0 )  
  return 1;
 else
  best_sum_sq_res=100*max_chi_square;
*/
 


 double var_max_freq=0.0;
 double var_min_freq=9999999;
 
 
// fprintf(stderr,"MAX_PEAK_FLUX=%lf\n",MAX_PEAK_FLUX);       
 /* Choose min peak flux */
 double var_min_peak_flux=9999999;
 double var_max_peak_flux=0.0;
 for(i=0;i<N_freq;i++){
  //!!
  //fprintf(stderr,"A %lf\n",freq[i]);
  //!!
  if( observed_flux[i]<var_min_peak_flux )var_min_peak_flux=observed_flux[i];
  if( observed_flux[i]>var_max_peak_flux )var_max_peak_flux=observed_flux[i];
  if( freq[i]>var_max_freq )var_max_freq=freq[i];
  if( freq[i]<var_min_freq )var_min_freq=freq[i];
  // positive test
  if( observed_flux[i]<0.0 )return 5;
//  fprintf(stderr,"%lf %lf\n",freq[i],observed_flux[i]);
 }
 var_min_freq=var_min_freq/MIN_OBS_FLUX_TO_MIN_PEAK_FLUX;
 //!!
// fprintf(stderr,"MAX_PEAK_FLUX=%lf\n",MAX_PEAK_FLUX);
 var_max_freq=var_max_freq/MAX_OBS_FLUX_TO_MAX_PEAK_FLUX;

 double numax_min=var_max_freq;
 double numax_max=var_min_freq;

 double imax_min=MAX_PEAK_FLUX;
// fprintf(stderr,"MAX_PEAK_FLUX=%lf (imax_min=%lf)\n",MAX_PEAK_FLUX,imax_min);
 double imax_max=var_min_peak_flux;

 double gamma_min=MAX_GAMMA;
 double gamma_max=MIN_GAMMA;

// fprintf(stderr,"Govorit fit_synchrotron, NACHINAEM, verbose=%d\n",verbose);

 /* get first guess for best_sum_sq_res */
 int j;
 double random01;


 if(verbose==1)fprintf(stderr,"Starting synchrotron fit\n");

 /* Initial guess */
 for(gamma=MIN_GAMMA;gamma<MAX_GAMMA;gamma+=0.2)
  for(numax=var_min_freq;numax<var_max_freq;numax+=BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP*numax){
   sum_sq_res=0;
 //  gamma=2.5;
   imax=var_max_peak_flux;
   synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
   for(i=0;i<N_freq;i++){
    if( observed_flux[i]!=0.0 ){
     sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
     if( sum_sq_res>best_sum_sq_res )break;
    }
   }// i (freq)
   if( sum_sq_res<best_sum_sq_res ){
    best_sum_sq_res=sum_sq_res;
    (*best_gamma)=gamma;
    (*best_numax)=numax;
    (*best_imax)=imax;
   }
  }
 if(verbose==1)fprintf(stderr,"Best initial guess:\n");
 if(verbose==1)fprintf(stderr,"nu_max: %lf %lf %lf\n",var_min_freq,(*best_numax),var_max_freq);
 if(verbose==1)fprintf(stderr,"peak_flux: %lf %lf %lf\n",var_min_peak_flux,(*best_imax),MAX_PEAK_FLUX);
 if(verbose==1)fprintf(stderr,"gamma: %lf %lf %lf\n",MIN_GAMMA,(*best_gamma),MAX_GAMMA);

 /* Set max peak flux */
 var_max_peak_flux=5*var_max_peak_flux;

 /* Try Monte Carlo */ 
 init_random_generator();
 for(j=0;j<10000;j++){
  sum_sq_res=0;
  random01=(double)random()/(double)RAND_MAX;
  gamma=MIN_GAMMA+random01*(MAX_GAMMA-MIN_GAMMA);
  imax=var_min_peak_flux+random01*(var_max_peak_flux-var_min_peak_flux);
  numax=var_min_freq+random01*(var_max_freq-var_min_freq);
  synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
  for(i=0;i<N_freq;i++){
   if( observed_flux[i]!=0.0 ){
    sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
    if( sum_sq_res>best_sum_sq_res )break;
   }
  }// i (freq)
  if( sum_sq_res<best_sum_sq_res ){
   best_sum_sq_res=sum_sq_res;
   (*best_gamma)=gamma;
   (*best_numax)=numax;
   (*best_imax)=imax;
  }
//!  if(verbose==1)fprintf(stderr,"Monte Carlo - parameters improved\n");

//  fprintf(stderr,"best_sum_sq_res=%lf\n",best_sum_sq_res);
 }

 
 // if best guess > minimal obs. freq - set var_min_freq = minimal obs. freq 
 if( (*best_numax)>BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP*(*best_numax)+MIN_OBS_FLUX_TO_MIN_PEAK_FLUX*var_min_freq )var_min_freq=var_min_freq*MIN_OBS_FLUX_TO_MIN_PEAK_FLUX;
 // if best guess < minimal obs. freq - set var_max_freq = minimal obs. freq * 2 
// if( (*best_numax)<MIN_OBS_FLUX_TO_MIN_PEAK_FLUX*var_min_freq-BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP )var_max_freq=var_min_freq*MIN_OBS_FLUX_TO_MIN_PEAK_FLUX;

 if(verbose==1)fprintf(stderr,"Best guess with Monte Carlo: \n");
 if(verbose==1)fprintf(stderr,"nu_max: %lf %lf %lf\n",var_min_freq,(*best_numax),var_max_freq);
 if(verbose==1)fprintf(stderr,"peak_flux: %lf %lf %lf\n",var_min_peak_flux,(*best_imax),var_max_peak_flux);
 if(verbose==1)fprintf(stderr,"gamma: %lf %lf %lf\n",MIN_GAMMA,(*best_gamma),MAX_GAMMA);


  
/* fprintf(stderr,"var_min_freq=%lf ",var_min_freq);
 fprintf(stderr,"var_max_freq=%lf ",var_max_freq);
 fprintf(stderr,"var_min_peak_flux=%lf ",var_min_peak_flux);
 fprintf(stderr,"MAX_PEAK_FLUX=%lf (imax_min=%lf)",MAX_PEAK_FLUX,imax_min);
 fprintf(stderr,"MIN_GAMMA=%lf ",MIN_GAMMA);
 fprintf(stderr,"MAX_GAMMA=%lf \n",MAX_GAMMA);
*/       
 /* First pass with big steps */
 for(numax=var_min_freq;numax<var_max_freq;numax+=BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP*numax){
  for(imax=var_min_peak_flux;imax<var_max_peak_flux;imax+=BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP){
   for(gamma=MIN_GAMMA;gamma<MAX_GAMMA;gamma+=BIG_STEP_TO_SMALL_STEP*GAMMA_STEP){
    //fprintf(stderr,"Inner circle\n");
    sum_sq_res=0;
    synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
    for(i=0;i<N_freq;i++){
     if( observed_flux[i]!=0.0 ){
      sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
      if(sum_sq_res>best_sum_sq_res)break;
     }
    }
//    fprintf(stderr,"Inner circle  %lf\n",sum_sq_res);
    if( sum_sq_res<best_sum_sq_res ){
     best_sum_sq_res=sum_sq_res;
     (*best_gamma)=gamma;
     (*best_numax)=numax;
     (*best_imax)=imax;  
     //if(verbose==1)fprintf(stderr,"1 best_gamma=%5.3lf best_numax=%6.3lf best_imax=%6.3lf\n",(*best_gamma),(*best_numax),(*best_imax));
     //if(verbose==1)fprintf(stderr,"BIG STEP - parameters improved\n");
    }
   } 
  }  
 }

/*
 if( max_chi_square<=0.0 )  
  return 1;
 else
  best_sum_sq_res=2*max_chi_square;
*/
 
 /* Second pass - fine tuning */
 for(numax=(*best_numax)-1*BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP*(*best_numax);numax<(*best_numax)+1*BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP*(*best_numax);numax+=PEAK_FREQ_STEP*numax){
  for(imax=(*best_imax)-1*BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP;imax<(*best_imax)+1*BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP;imax+=PEAK_FLUX_STEP){
   for(gamma=(*best_gamma)-1*BIG_STEP_TO_SMALL_STEP*GAMMA_STEP;gamma<(*best_gamma)+1*BIG_STEP_TO_SMALL_STEP*GAMMA_STEP;gamma+=GAMMA_STEP){
    sum_sq_res=0;
    synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
    for(i=0;i<N_freq;i++){
     if( observed_flux[i]!=0.0 ){
      sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
      if(sum_sq_res>best_sum_sq_res)break;
     }
    }
    if( sum_sq_res<best_sum_sq_res ){
     best_sum_sq_res=sum_sq_res;
     (*best_gamma)=gamma;
     (*best_numax)=numax;
     (*best_imax)=imax;  
     //if(verbose==1)fprintf(stderr,"2 best_gamma=%5.3lf best_numax=%6.3lf best_imax=%6.3lf\n",(*best_gamma),(*best_numax),(*best_imax));
     //if(verbose==1)fprintf(stderr,"SMALL STEP - parameters improved\n");
    }
   } 
  }  
 }   

 /* 3rd pass - monte carlo around best fit */
  
 for(j=0;j<10000;j++){
  sum_sq_res=0;
  random01=(double)random()/(double)RAND_MAX;
  gamma=(*best_gamma)-GAMMA_STEP+2*random01*GAMMA_STEP;//MIN_GAMMA+random01*(MAX_GAMMA-MIN_GAMMA);
  imax=(*best_imax)-PEAK_FLUX_STEP+2*random01*PEAK_FLUX_STEP;//var_min_peak_flux+random01*(MAX_PEAK_FLUX-var_min_peak_flux);
  numax=(*best_numax)-PEAK_FREQ_STEP*(*best_numax)+2*random01*PEAK_FREQ_STEP*(*best_numax);//var_min_freq+random01*(var_max_freq-var_min_freq);
  synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
  for(i=0;i<N_freq;i++){
   if( observed_flux[i]!=0.0 ){
    sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
    if( sum_sq_res>best_sum_sq_res )break;
   }
  }// i (freq)
  if( sum_sq_res<best_sum_sq_res ){
   best_sum_sq_res=sum_sq_res;
   (*best_gamma)=gamma;
   (*best_numax)=numax;
   (*best_imax)=imax;
   if(verbose==1)fprintf(stderr,"Fine tuning with Monte Carlo - parameters improved\n");
  }
//  fprintf(stderr,"best_sum_sq_res=%lf\n",best_sum_sq_res);
 }

 /* Stop timer */
 end_time=time(NULL);
 elapsed_time=difftime(end_time, start_time);
 
 if(verbose==1)fprintf(stderr,"Fitting limits: \n");
 if(verbose==1)fprintf(stderr,"nu_max: %lf %lf %lf\n",var_min_freq,(*best_numax),var_max_freq);
 if(verbose==1)fprintf(stderr,"peak_flux: %lf %lf %lf\n",var_min_peak_flux,(*best_imax),var_max_peak_flux);
 if(verbose==1)fprintf(stderr,"gamma: %lf %lf %lf\n",MIN_GAMMA,(*best_gamma),MAX_GAMMA);
 if(verbose==1)fprintf(stderr,"Computation time: %.0lf sec\n",elapsed_time);               
                                                                       
 if(verbose==1)fprintf(stderr,"SYNCHROTRON FIT:  gamma=%5.3lf  nu_peak=%6.3lf [GHz]  S_peak=%6.3lf [Jy] ",(*best_gamma),(*best_numax),(*best_imax));

 // !!! NO LIMITS, JUST DROP IT !!!
 if( (*best_numax)<var_min_freq )
  return 1;

 /* 1st check aren't we outside (or too close to) the fitting limits */
 if( (*best_numax)<var_min_freq+PEAK_FREQ_STEP*(*best_numax) && (*best_gamma)>1.0 ){
//  fprintf(stderr,"set_synchrotron_limits()\n");
  //(*best_gamma)=var_min_freq;
  set_synchrotron_limits( best_gamma, best_numax, best_imax, freq, observed_flux, w, N_freq);
  return 0;
 }


 /* 2nd check aren't we outside (or too close to) the fitting limits */
 if( fabs((*best_numax)-var_min_freq)<=PEAK_FREQ_STEP*(*best_numax) || fabs((*best_numax)-var_max_freq)<=PEAK_FREQ_STEP*(*best_numax) || fabs((*best_imax)-var_min_peak_flux)<=PEAK_FLUX_STEP || fabs((*best_imax)-var_max_peak_flux)<=PEAK_FLUX_STEP  || fabs((*best_gamma)-MIN_GAMMA)<=GAMMA_STEP || fabs((*best_gamma)-MAX_GAMMA)<=GAMMA_STEP ){
//  fprintf(stderr,"set_synchrotron_limits()\n");
//  set_synchrotron_limits( best_gamma, best_numax, best_imax, freq, observed_flux, w, N_freq);
  if(verbose==1)fprintf(stderr,"BAD FIT (2nd check - outside (or too close to) the fitting limits)\n");
  if(verbose==1)fprintf(stderr,"SYNCHROTRON FIT:  gamma=%5.3lf  nu_peak=%6.3lf [GHz]  S_peak=%6.3lf [Jy] ",(*best_gamma),(*best_numax),(*best_imax));
  
//  for(i=0;i<N_freq;i++){
//   fprintf(stderr,"%lf %lf  %lf\n",freq[i],observed_flux[i],1.0/sqrt(w[i]));
//  }
//  exit(1);
  
  return 3;
 }


 /* Check if the synchrotron fit is any better then power law fit */

//!!!
 if( best_sum_sq_res==2*max_chi_square ){
  fprintf(stderr,"best_sum_sq_res==max_chi_square\n");
  return 2;
 }

 (*chi_square)=best_sum_sq_res;
 if( best_sum_sq_res>max_chi_square ){
  if(verbose==1)fprintf(stderr,"BAD FIT\n");
  if(verbose==1)fprintf(stderr,"max_chi_square>2*max_chi_square ");
  if( (*best_gamma)>2.0 ){
//   fprintf(stderr,"set_synchrotron_limits()\n");
//   (*best_gamma)=var_min_freq;
//   set_synchrotron_limits( best_gamma, best_numax, best_imax, freq, observed_flux, w, N_freq);
   return 1;
  }
  else
   return 2;
 }
 
 
// fprintf(stderr,"\n");

 return 0; // !!!!!!!!!!!!!!!! Zatychka !!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 /* Error estimation */
/*
// for(numax=(*best_numax)-5*BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP;numax<(*best_numax)+5*BIG_STEP_TO_SMALL_STEP*PEAK_FREQ_STEP;numax+=PEAK_FREQ_STEP){
 for(numax=MIN_PEAK_FREQ;numax<MAX_PEAK_FREQ;numax+=PEAK_FREQ_STEP){
//  for(imax=(*best_imax)-5*BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP;imax<(*best_imax)+5*BIG_STEP_TO_SMALL_STEP*PEAK_FLUX_STEP;imax+=PEAK_FLUX_STEP){
  for(imax=var_min_peak_flux;imax<MAX_PEAK_FLUX;imax+=PEAK_FLUX_STEP){
   // gamma is fixed
   //gamma=(*best_gamma);
   for(gamma=MIN_GAMMA;gamma<MAX_GAMMA;gamma+=GAMMA_STEP){
   //for(gamma=(*best_gamma)-500*GAMMA_STEP;gamma<(*best_gamma)+500*GAMMA_STEP;gamma+=BIG_STEP_TO_SMALL_STEP*GAMMA_STEP){
    sum_sq_res=0;
    synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
    for(i=0;i<N_freq;i++){
     sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
     if(sum_sq_res>best_sum_sq_res+3.5)break;
    }
    if( sum_sq_res<best_sum_sq_res+3.5 ){
     if( numax<numax_min )numax_min=numax;
     if( numax>numax_max )numax_max=numax;
     if( imax<imax_min )imax_min=imax;
     if( imax>imax_max )imax_max=imax;
     if( gamma<gamma_min )gamma_min=gamma;
     if( gamma>gamma_max )gamma_max=gamma;
    }
   }
  }
 }
*/
 /* Quick error estimation */

 /* nu */
 gamma=(*best_gamma);
 numax=(*best_numax);
 imax=(*best_imax);
 for(numax=var_min_freq;numax<var_max_freq;numax+=PEAK_FREQ_STEP){
  sum_sq_res=0;
  synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
  for(i=0;i<N_freq;i++){
   sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
   if(sum_sq_res>best_sum_sq_res+3.5)break;
  }
  if( sum_sq_res<best_sum_sq_res+3.5 ){
   if( numax<numax_min )numax_min=numax;
   if( numax>numax_max )numax_max=numax;
   if( imax<imax_min )imax_min=imax;
   if( imax>imax_max )imax_max=imax;
   if( gamma<gamma_min )gamma_min=gamma;
   if( gamma>gamma_max )gamma_max=gamma;
  }  
 }

 /* imax */
 gamma=(*best_gamma);
 numax=(*best_numax);
 imax=(*best_imax);
 for(imax=var_min_peak_flux;imax<MAX_PEAK_FLUX;imax+=PEAK_FLUX_STEP){
  sum_sq_res=0;
  synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
  for(i=0;i<N_freq;i++){
   sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
   if(sum_sq_res>best_sum_sq_res+3.5)break;
  }
  if( sum_sq_res<best_sum_sq_res+3.5 ){
   if( numax<numax_min )numax_min=numax;
   if( numax>numax_max )numax_max=numax;
   if( imax<imax_min )imax_min=imax;
   if( imax>imax_max )imax_max=imax;
   if( gamma<gamma_min )gamma_min=gamma;
   if( gamma>gamma_max )gamma_max=gamma;
  }  
 }

 /* gamma */
 gamma=(*best_gamma);
 numax=(*best_numax);
 imax=(*best_imax);
 for(gamma=MIN_GAMMA;gamma<MAX_GAMMA;gamma+=GAMMA_STEP){
  sum_sq_res=0;
  synchrotron( gamma, numax, imax, freq, theor_flux, N_freq);
  for(i=0;i<N_freq;i++){
   sum_sq_res+=w[i]*(theor_flux[i]-observed_flux[i])*(theor_flux[i]-observed_flux[i]);
   if(sum_sq_res>best_sum_sq_res+3.5)break;
  }
  if( sum_sq_res<best_sum_sq_res+3.5 ){
   if( numax<numax_min )numax_min=numax;
   if( numax>numax_max )numax_max=numax;
   if( imax<imax_min )imax_min=imax;
   if( imax>imax_max )imax_max=imax;
   if( gamma<gamma_min )gamma_min=gamma;
   if( gamma>gamma_max )gamma_max=gamma;
  }  
 }

 if( fabs(numax_min-var_min_freq)<=PEAK_FREQ_STEP || fabs(numax_max-var_max_freq)<=PEAK_FREQ_STEP || fabs(imax_min-var_min_peak_flux)<=PEAK_FLUX_STEP || fabs(imax_max-MAX_PEAK_FLUX)<=PEAK_FLUX_STEP  || fabs(gamma_min-MIN_GAMMA)<=GAMMA_STEP || fabs(gamma_max-MAX_GAMMA)<=GAMMA_STEP ){
  if(verbose==1)fprintf(stderr,"BAD FIT: parameter valuees are too close to borders\n");
  if(verbose==1)fprintf(stderr,"SYNCHROTRON:  gamma=%5.3lf (%5.3lf - %5.3lf) nu_peak=%6.3lf (%6.3lf - %6.3lf)  S_peak=%6.3lf (%6.3lf - %6.3lf)  ",(*best_gamma), gamma_min, gamma_max,(*best_numax), numax_min, numax_max,(*best_imax), imax_min, imax_max);
  return 4;
 }
// fprintf(stderr,"\n");

 if( fabs(gamma_min-(*best_gamma))>0.1*(*best_gamma) || fabs(gamma_max-(*best_gamma))>0.1*(*best_gamma) || fabs(numax_max-(*best_numax))>0.1*(*best_numax) || fabs(numax_min-(*best_numax))>0.1*(*best_numax) || fabs(imax_min-(*best_imax))>0.1*(*best_imax) || fabs(imax_max-(*best_imax))>0.1*(*best_imax) ){
  if(verbose==1)fprintf(stderr,"BAD FIT: parameter errors are >10\% \n");
  if(verbose==1)fprintf(stderr,"SYNCHROTRON:  gamma=%5.3lf (%5.3lf - %5.3lf) nu_peak=%6.3lf (%6.3lf - %6.3lf)  S_peak=%6.3lf (%6.3lf - %6.3lf)  ",(*best_gamma), gamma_min, gamma_max,(*best_numax), numax_min, numax_max,(*best_imax), imax_min, imax_max);
  return 5;
 }
// fprintf(stderr,"\n");
 
 
 if(verbose==1)fprintf(stderr,"SYNCHROTRON:  gamma=%5.3lf (%5.3lf - %5.3lf) nu_peak=%6.3lf (%6.3lf - %6.3lf)  S_peak=%6.3lf (%6.3lf - %6.3lf)  ",(*best_gamma), gamma_min, gamma_max,(*best_numax), numax_min, numax_max,(*best_imax), imax_min, imax_max);
 /* End of error estimation */
 
 if(verbose==1)fprintf(stderr,"Synchrotron:  Chi^2 = %lf\n", best_sum_sq_res);

// fprintf(stderr,"Govorit fit_synchrotron, ZAKANCHIVAEM, verbose=%d\n",verbose);
 
 return 0;
}
                                                                          
/* C*x^A*/
int powerfit( double *best_C, double *best_A, double *freq, double *observed_flux, double *w, int N_freq, double *max_chi_square, int verbose){
 double A,C,S;
 double best_S=99999999;
 int i;
 double computed_flux;
 
 double A_min = SP_INDEX_MAX;
 double A_max = SP_INDEX_MIN;

 double C_min, C_max;
 
 /* get first guess for best_S */
 int j;
 double random01;
 
 for(j=0;j<1000;j++){
  S=0;
  random01=(double)random()/(double)RAND_MAX;
  C=C_MIN+random01*(C_MAX-C_MIN);
  A=SP_INDEX_MIN+random01*(SP_INDEX_MAX-SP_INDEX_MIN);
//  fprintf(stderr,"A=%lf C=%lf random01=%lf  RAND_MAX=%d\n",A,C,random01,RAND_MAX);
  for(i=0;i<N_freq;i++){
   if( observed_flux[i]!=0.0 ){
    computed_flux=pow(10,C)*pow(freq[i],A);
    S+=w[i]*(observed_flux[i]-computed_flux)*(observed_flux[i]-computed_flux);
    if( S>best_S )break;
   }
  }// i (freq)
  if( S<best_S )best_S=S;
//  fprintf(stderr,"best_S=%lf\n",best_S);
 }
 
 for(A=SP_INDEX_MIN;A<=SP_INDEX_MAX;A+=SP_INDEX_STEP){
  for(C=C_MIN;C<=C_MAX;C+=C_STEP){
   S=0;
   for(i=0;i<N_freq;i++){
    if( observed_flux[i]!=0 ){
     computed_flux=pow(10,C)*pow(freq[i],A); 
     S+=w[i]*(observed_flux[i]-computed_flux)*(observed_flux[i]-computed_flux);
     if( S>best_S )break;
    }
   }// i (freq)
   if( S<best_S ){
    best_S=S;
    (*best_C)=C;
    (*best_A)=A;
   }
//   else{
//    if( S<best_S+2.8 ){
//     if( A>A_max )A_max=A;
//     if( A<A_min )A_min=A;
//    }
//   }
 
   
//   fprintf(stderr,"C=%lf A=%lf   best_C=%lf, best_A=%lf\n",C,A,(*best_C),(*best_A));
  }// C
 }// A
 
 (*max_chi_square)=best_S;
 
// fprintf(stderr,"powerfit(): returning (*max_chi_square)=%lf\n",(*max_chi_square));
 
 if( fabs((*best_A)-SP_INDEX_MIN)<=SP_INDEX_STEP || fabs((*best_A)-SP_INDEX_MAX)<=SP_INDEX_STEP || fabs((*best_C)-C_MIN)<=C_STEP || fabs((*best_C)-C_MAX)<=C_STEP ){
  if(verbose==1)fprintf(stderr,"ERROR! Fitting faild.\n");
  if(verbose==1)fprintf(stderr,"A %lf %lf %lf    B %lf %lf %lf\n",(*best_A), SP_INDEX_MIN, SP_INDEX_MAX,  (*best_C), C_MIN, C_MAX);
   //exit(1);
  //(*max_chi_square)=0.0;
  return 1;
 }


 return 0; // !!!!!!!!!!!!!!!! Zatychka !!!!!!!!!!!!!!!!!!!!!!!!!!!!

 /// =0 !!!!!!!!!!!!!

 C_min=C_max=(*best_C);

 /* Error estimation */
 for(A=SP_INDEX_MIN;A<=SP_INDEX_MAX;A+=SP_INDEX_STEP){
  for(C=C_MIN;C<=C_MAX;C+=C_STEP){
   S=0;
   for(i=0;i<N_freq;i++){
    computed_flux=pow(10,C)*pow(freq[i],A);
    S+=w[i]*(observed_flux[i]-computed_flux)*(observed_flux[i]-computed_flux);
    if( S>best_S+2.8 )break;
   }// i (freq)
   if( S<best_S+2.8 ){
    if( A>A_max )A_max=A;
    if( A<A_min )A_min=A;
    if( C>C_max )C_max=C;
    if( C<C_min )C_min=C;
   }
  }// C
 }// A
 /* End of error estimation */  
 if(verbose==1)fprintf(stderr,"  A %lf %lf %lf    B %lf %lf %lf\n",(*best_A), SP_INDEX_MIN, SP_INDEX_MAX,  (*best_C), C_MIN, C_MAX);
 if(verbose==1)fprintf(stderr,"  C = %lf (%lf - %lf) ", (*best_C), C_min, C_max );
 if(verbose==1)fprintf(stderr,"  A = %lf (%lf - %lf) ", (*best_A), A_min, A_max);
 if(verbose==1)fprintf(stderr,"Power law:  Chi^2 = %lf\n",best_S);

 return 0;
}

/* The following formulas are from Pacholczyk, 1970 */

double gammafunc(double x){
// fprintf(stderr,"GAMMA(%lg)\n",x);
 if( x==-1 || x==-2 || x==-3 || x==-4 || x==-5 )x+=0.001;
 return gsl_sf_gamma(x);
}

double c(){
 return 29979245800.0; // c (cm/sec)
}

double c1(){
 return 6.27*pow(10,18);
}

double c3(){
 return 1.87*pow(10,-23);
}

double c5(double gamma){
// fprintf(stderr,"c5(%lg)\n",gamma);
 return 0.25*c3()*gammafunc( (3*gamma-1)/12 )*gammafunc( (3*gamma+7)/12 )*gammafunc( (gamma+7/3)/(gamma+1) );
}

double c6(double gamma){
// fprintf(stderr,"c6(%lg)\n",gamma);
 return 1.0/32.0*( c()/c1() )*( c()/c1() )*c3()*(gamma+10/3)*gammafunc( (3*gamma+2)/12 )*gammafunc( (3*gamma+10)/12 );
}

float f_H_cgs(float fnu_max, float fSmax, float falpha, double bmaj_deg, double bmin_deg){
 double beam_area_ster=M_PI*(bmaj_deg*3600/206264.8)*(bmin_deg*3600/206264.8)/(4*log(2));
// fprintf(stderr,"beam_area_ster=%lg test=2.3504e-17\n",beam_area_ster);
 double nu_max=(double)fnu_max*1e9; // GHz to Hz
 double Smax=(double)fSmax;
 double alpha=(double)falpha;
 double gamma=1-2*alpha;
 double taumax=isolve(gamma);
 double nu1=pow(taumax,2/(gamma+4))*nu_max;
// fprintf(stderr,"nu1=%lf GHz\n",nu1/1e9);
 double nu[3]={10*nu_max, 5*nu_max, 2*nu_max};
 double I_nu[3];
// fprintf(stderr,"f_H_cgs(%g, %g, %g)\n",fnu_max,fSmax,falpha);
 synchrotron( gamma, nu_max, Smax, nu, I_nu, 1);
// fprintf(stderr,"I_nu[0]=%lf Jy ",I_nu[0]);
 I_nu[0]=I_nu[0]*pow(10,-23)/beam_area_ster; // Jy to cgs // !!! / beam solid angle
// fprintf(stderr,"=%lg cgs ( / beam solid angle )\n",I_nu[0]);
 double S_nu1=I_nu[0]/J(nu[0]/nu1, gamma);
 double H;

 double c5_=c5(gamma);
 double c6_=c6(gamma);
 
// fprintf(stderr,"c5=%lg c6=%lg\n",c5_,c6_);
 H=c5_*c5_/(c6_*c6_)*pow(2*c1(),-5)/(S_nu1*S_nu1)*pow(nu1,5);
// fprintf(stderr,"H=%lg\n",H);

/*
 H=c5(gamma)*c5(gamma);
 fprintf(stderr,"H=%lf\n",H);
 H=H/(c6(gamma)*c6(gamma));
 fprintf(stderr,"H=%lf\n",H);
 H=H*pow(2*c1(),-5)/(S_nu1*S_nu1)*pow(nu1,5);
 fprintf(stderr,"H=%lf\n",H);
*/

 return (float)H;
}
